# -*- coding: utf-8 -*-

# Meta

__version__ = "0.0.2"
__author__ = 'Rhys Elsmore'
__email__ = 'me@rhys.io'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2014 Rhys Elsmore'

# Module Namespace

from .core import LogMetrics
from .api import timer, increment, sample, measure, unique




# -*- coding: utf-8 -*-

"""
log_metrics.core
~~~~~~~~~~~~~~~~~

This module implements the basic interfaces for our log-metrics package.
"""


import time
import logging
from functools import wraps
import sys

EXC = (None, None, None)


class ContextDecorator(object):
    before = None
    after = None

    def __call__(self, f):
        @wraps(f)
        def inner(*args, **kw):
            if self.before is not None:
                self.before()
            exc = EXC
            try:
                result = f(*args, **kw)
            except Exception:
                exc = sys.exc_info()
            catch = False
            if self.after is not None:
                catch = self.after(*exc)
            if not catch and exc is not EXC:
                _reraise(*exc)
            return result
        return inner

    def __enter__(self):
        if self.before is not None:
            return self.before()

    def __exit__(self, *exc):
        catch = False
        if self.after is not None:
            catch = self.after(*exc)
        return catch


class Timer(ContextDecorator):
    def __init__(self, name, source, metrics):
        self.name = name
        self.metrics = metrics
        self.source = source

    def before(self):
        self.start = time.time()

    def after(self, *exc):
        duration = (time.time() - self.start)*1000
        self.metrics.measure("%s.ms" % self.name, "%.2f" % duration, self.source)


class LogMetrics(object):

    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        if not self.logger.handlers:
            handler = logging.StreamHandler(sys.stdout)
            handler.setFormatter(logging.Formatter('%(message)s'))
            self.logger.addHandler(handler)

    def _log(self, source, prefix, name, value):
        val = "%s#%s=%s" % (prefix, name, value)
        if source:
            val = "source=%s %s" % (source, val)
        self.logger.info(val)

    def timer(self, name, source=None):
        return Timer(name, source, self)

    def increment(self, name, val=None, source=None):
        val = val or 1
        self._log(source, "count", name, val)

    def sample(self, name, val, source=None):
        self._log(source, "sample", name, val)

    def measure(self, name, val, source=None):
        self._log(source, "measure", name, val)

    def unique(self, name, val, source=None):
        self._log(source, "unique", name, val)



#!/usr/bin/env python

import os
import sys

import log_metrics

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

packages = [
    'log_metrics',
]

requires = []


with open('README.rst') as f:
    readme = f.read()
with open('HISTORY.rst') as f:
    history = f.read()


setup(
    name='log-metrics',
    version=log_metrics.__version__,
    description='Python Metric Logger.',
    long_description=readme + '\n\n' + history,
    author=log_metrics.__author__,
    author_email=log_metrics.__email__,
    url='https://github.com/rhyselsmore/log-metrics',
    packages=packages,
    package_data={'': ['LICENSE']},
    package_dir={'log_metrics': 'log_metrics'},
    include_package_data=True,
    install_requires=requires,
    license='Apache 2.0',
    zip_safe=False,
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',

    ),
)



# -*- coding: utf-8 -*-

"""
log_metrics.api
~~~~~~~~~~~~~~~~~

This module implements the log-metrics API.
"""

from .core import LogMetrics


def timer(name, source=None):
    return LogMetrics().timer(name, source)


def increment(name, val=None, source=None):
    return LogMetrics().increment(name, val, source)


def sample(name, val, source=None):
    return LogMetrics().sample(name, val, source)


def measure(name, val, source=None):
    return LogMetrics().measure(name, val, source)


def unique(name, val, source=None):
    return LogMetrics().unique(name, val, source)
